@@ -1,5 +1,23 @@
 #!/usr/bin/python
 #-*- coding: utf-8 -*-
+
+# Copyright (c) 2013 Eric F <efigue> Figerson
+# Author(s):
+#   Eric F <eric.foss@free.fr>
+
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 import os
 import sys
 import csv
@@ -12,22 +30,19 @@ import re
 import glob
 import datetime
 import platform
-def get_script_dir():
-    return os.path.join(os.path.dirname(__file__), '..')
-sys.path.append(get_script_dir())
-import config
-from i18n import _
-
-def get_data_dir():
-    return config.get_internal_data_dir()
+import facct.config as config
+from facct.i18n import _
 
 def setlocale():
     local_name = 'fr_FR.UTF-8'
     if platform.os.name == 'nt':
        #See: http://msdn.microsoft.com/en-us/library/cdax410z%28VS.71%29.aspx
        local_name = 'fra_fra'
-    if local_name.split('.')[0].lower() in locale.locale_alias:
-        locale.setlocale(locale.LC_ALL, local_name)
+       if 'fr_fr' in locale.locale_alias:
+           locale.setlocale(locale.LC_ALL, local_name)
+    else:
+        if local_name.split('.')[0].lower() in locale.locale_alias:
+            locale.setlocale(locale.LC_ALL, local_name)
 setlocale()
 
 
@@ -65,19 +80,26 @@ def check_output(p):
     if p.stderr:
         if p.stdout:
             print (''.join(p.stdout))
-        print (''.join(p.sterr))
+        print (''.join(p.stderr))
     return (0 == p.returncode)
 
 def gen_dvi(dico):
-    if not os.path.exists(dico['root_file'] + '.tex'): 
+    if not os.path.exists(dico['root_file'] + '.tex'):
         sys.stderr.write(_('%(root_file)s.tex: file not found\n') % dico)
         sys.exit(2)
         return False
-    p = subprocess.Popen('latex -halt-on-error %(base_file)s.tex' % dico, 
+    p = subprocess.Popen('latex -version', shell=True, stdout=subprocess.PIPE)
+    p.communicate()
+    if p.returncode:
+        sys.stderr.write(_('latex command not found, please install' +\
+            ' (MikTeX on Windows, texlive-latex on Linux)\n'))
+        return False
+    p = subprocess.Popen('latex -halt-on-error %(base_file)s.tex' % dico,
         cwd=dico['tmp_dir'], shell=True, stdout=subprocess.PIPE)
     p.communicate()
     rc = check_output(p)
-    p.stdout.close()
+    if p.stdout:
+        p.stdout.close()
     if not rc:
         log_file = dico['root_file'] + '.log'
         if os.path.exists(log_file):
@@ -85,9 +107,10 @@ def gen_dvi(dico):
                 print (l)
         sys.exit(2)
     print (_('%(base_file)s.dvi generated.') % dico)
+    return True
 
 def gen_pdf(dico):
-    if not os.path.exists(dico['root_file']+'.dvi'): 
+    if not os.path.exists(dico['root_file']+'.dvi'):
         sys.stderr.write(_('%(root_file)s.dvi: file not found\n') % dico)
         sys.exit(2)
         return False
@@ -110,8 +133,8 @@ def gen_pdf(dico):
 def write_pdf(orga, year, month, company_name, unicode_TeX):
     year_dir = os.path.join(get_db_path(orga), str(year))
     tmp_dir = os.path.join(get_db_path(orga), 'tmp')
-    dico = { 'year_dir':year_dir, 
-        'id_bill' : '0'*(2-len(str(month))) + '%d%d' % (month, year), 
+    dico = { 'year_dir':year_dir,
+        'id_bill' : '0'*(2-len(str(month))) + '%d%d' % (month, year),
         'company_name' : company_name,
         'tmp_dir' : tmp_dir,
     }
@@ -133,14 +156,15 @@ def write_pdf(orga, year, month, company_name, unicode_TeX):
     open(os.path.join(tmp_dir, '%(base_file)s.tex' % dico), 'w').write(TeX_bill)
     print (_('%(base_file)s.tex generated.') % dico)
 
-    tmpl_file = os.path.join(get_data_dir(), '%(bill_tmpl)s.tex' % dico)
+    tmpl_file = os.path.join(config.get_internal_data_dir(),
+            '%(bill_tmpl)s.tex' % dico)
     if not os.path.exists(tmpl_file):
         sys.stderr.write(_('%s: file not found\n') % tmpl_file)
         sys.exit(2)
     shutil.copy(tmpl_file, tmp_dir)
 
-    gen_dvi(dico)
-    gen_pdf(dico)
+    if gen_dvi(dico):
+        gen_pdf(dico)
 
     for f in glob.glob(os.path.join(tmp_dir, '*')):
         os.remove(f)
@@ -150,7 +174,8 @@ def getDate(val):
     return datetime.datetime(*(time.strptime(val, '%Y/%m/%d')[0:6])).date()
 
 def getSameDayNextMonth(date):
-    return datetime.date(date.year + date.month//12, (date.month%12)+1, date.day)
+    return datetime.date(date.year + date.month//12,
+            (date.month % 12) + 1, date.day)
 
 def get_db_path(orga):
     return  config.get_bills_dir(orga)
@@ -183,7 +208,7 @@ _month = %(month)s
 _client = %(client)s
 _day_price = %(day_price)s
 _amendment = %(amendment)s
-_nb_bill = %(nb_bill)s 
+_nb_bill = %(nb_bill)s
 ''' % { 'year' : self._year,
         'month' : self._month,
         'client' : self._client,
@@ -219,7 +244,7 @@ _nb_bill = %(nb_bill)s
             print ('self.client=%s'%self._client)
             print ('self.day_price=%s'%self._day_price)
             print ('self.amendment=%s'%self._amendment)
-        
+
     def getBillNb(self):
         CLIENT, START_TIME, STOP_TIME, AMOUNT, AMENDMENT = range(5)
         min_time = None
@@ -274,16 +299,16 @@ _nb_bill = %(nb_bill)s
         sep=''
         if len(self._day_price) == 1:
             sep=' '
-        return '+'.join ([ 
-            '%s%s*%s%s' % (tjm[2],sep,sep,tjm[1]) for tjm in self._day_price]) 
+        return '+'.join ([
+            '%s%s*%s%s' % (tjm[2],sep,sep,tjm[1]) for tjm in self._day_price])
 
     def get_nb_days(self):
-        return sum ([ tjm[2] for tjm in self._day_price]) 
+        return sum ([ tjm[2] for tjm in self._day_price])
 
     def get_total_excluded_tax(self):
         if not self._total_excluded_tax:
             self._total_excluded_tax = sum (
-                    [ tjm[1]*tjm[2] for tjm in self._day_price]) 
+                    [ tjm[1]*tjm[2] for tjm in self._day_price])
         return self._total_excluded_tax
 
     def get_VAT(self):
@@ -298,14 +323,14 @@ _nb_bill = %(nb_bill)s
         return round(self._total_excluded_tax + self.get_VAT(),2)
 
     def get_dico(bill):
-        return { 'year' : bill._year, 
+        return { 'year' : bill._year,
             'month name': get_month_name(bill._year, bill._month),
-            'month id'  : '0'*(2-len(str(bill._month))) + str(bill._month), 
-            'last day'  : sorted(bill._dico_prestation.keys())[-1], 
+            'month id'  : '0'*(2-len(str(bill._month))) + str(bill._month),
+            'last day'  : sorted(bill._dico_prestation.keys())[-1],
             'nb days'   : bill.get_nb_days(),
             'nb bill'   : '0'*(6-len(str(bill._nb_bill))) +str(bill._nb_bill),
             'total VAT' : bill.get_total_excluded_tax(),
-            'VAT'       : '%.2f' % bill.get_VAT(), 
+            'VAT'       : '%.2f' % bill.get_VAT(),
             'total'     : '%.2f' % bill.get_total_tax_incl(),
             'calculus'  : bill.get_calculus(),
             'amendment' : bill._amendment,
@@ -314,6 +339,11 @@ _nb_bill = %(nb_bill)s
     def get_name(self):
         return 'bill'
 
+def get_encoding():
+    if platform.system().split('-')[0] in ('Windows', 'CYGWIN_NT'):
+        return 'windows-1252'
+    return 'utf-8'
+
 class TeX:
     def __init__(self):
         pass
@@ -324,15 +354,15 @@ class TeX:
             for d in week:
                 tex_weeks += '{{'
                 if bill._dico_prestation[d] == '0.5':
-                    tex_weeks += '%s (1/2)' % str(d) 
+                    tex_weeks += '%s (1/2)' % str(d)
                     tex_weeks += '}{'
                     tex_weeks += '*'
                 if bill._dico_prestation[d] == '1':
-                    tex_weeks += str(d) 
+                    tex_weeks += str(d)
                     tex_weeks += '}{'
                     tex_weeks += '*'
                 if bill._dico_prestation[d] == '0':
-                    tex_weeks += str(d) 
+                    tex_weeks += str(d)
                     tex_weeks += '}{'
                     tex_weeks += '\\ '
                 if bill._dico_prestation[d] == '-':
@@ -346,38 +376,35 @@ class TeX:
 
     def getVarDef(self, db):
         def getVarDef_fromVal(k, v):
-            return r'\gdef\%s%s{%s}' % (db.get_name(), 
+            return r'\gdef\%s%s{%s}' % (db.get_name(),
                     ''.join([w[0].upper()+w[1:] for w in k.split()]),
                     str(v).replace('_','\_'))
-        return '\n'.join([getVarDef_fromVal(k, v) for k,v in db.get_dico().items()])
+        return '\n'.join([
+                getVarDef_fromVal(k, v) for k,v in db.get_dico().items()])
 
-    def create_utf8_translation_table(self, utf8_file):
-        translation_table = {}
+    def create_utf8_translation_table(self):
+        tr_table = {}
+        utf8_file = os.path.join(config.get_internal_data_dir(), 'utf8ienc.dtx')
         for line in open(utf8_file):
             m = re.match(r'%.*\DeclareUnicodeCharacter\{(\w+)\}\{(.*)\}', line)
             if m:
                 codepoint, latex = m.groups()
                 latex = latex.replace('@tabacckludge', '')
-                if sys.version_info[0] == 2:
-                    latex = latex.decode('utf-8')
-                translation_table[int(codepoint, 16)] = latex
-        return translation_table
+                tr_table[int(codepoint, 16)] = decode(latex)
+        return tr_table
 
     def get_unicode_TeX(self, tex_data, dir_root):
-        utf8_file = os.path.join(get_data_dir(), 'utf8ienc.dtx')
-        translation_table = self.create_utf8_translation_table(utf8_file)
-        if sys.version_info[0] == 2:
-            if not platform.os.name == 'nt':
-                tex_data = tex_data.decode('utf-8')
-            if platform.os.name == 'posix':
-                #tex_data = tex_data.decode('utf-8')
-                pass
-            if platform.os.name == 'nt':
-                #tex_data = tex_data.decode('utf-8')
-                pass
-        return tex_data.translate(translation_table)
+        tr_table = self.create_utf8_translation_table()
+        if sys.version_info[0] >= 3:
+            return tex_data.translate(tr_table)
+        return decode(tex_data).translate(tr_table).encode(get_encoding())
 
 
+def decode(s):
+    if sys.version_info[0] >= 3:
+        return s
+    return s.decode(get_encoding())
+
 def get_month_name(year, month):
     month_name = datetime.date(year, month, 1).strftime('%B')
     month_name = month_name[0].upper() + month_name[1:]
@@ -387,8 +414,9 @@ def bill_creation(orga, bill, company, dir_root):
     tex = TeX()
     db_list = [ Client(orga, bill._client), bill, company, Bank(orga)]
     return tex.get_unicode_TeX(
-    '\n'.join([tex.calendar(bill)]+[tex.getVarDef(db) for db in db_list]), dir_root)
-        
+        '\n'.join([tex.calendar(bill)]+[tex.getVarDef(db) for db in db_list]),
+        dir_root)
+
 def init_bill(orga, csv_file_name):
     year, month = filename_info(csv_file_name)
     year_bill_dir = os.path.join(config.get_bills_dir(orga), str(year))
@@ -400,10 +428,10 @@ def init_bill(orga, csv_file_name):
         if day_of_week%7 > 4:
             return '0'
         return '1'
-            
+
     lines = []
     for w in calendar.monthcalendar(year, month):
-        lines.append(';'.join([ deco(dw, dm) for dw, dm in enumerate(w)]) + '\n')
+        lines.append(';'.join([ deco(dw, dm) for dw, dm in enumerate(w)]) +'\n')
     csv_file_name = os.path.join(year_bill_dir, os.path.basename(csv_file_name))
     open(csv_file_name, 'a').writelines(lines)
     return csv_file_name
@@ -419,12 +447,13 @@ class Company():
             if self._dico['form'] == 'SARL':
                 long_form = 'Société à Responsabilité Limitée'
             VAT_intra = self._dico['VAT code prefix'] + self._dico['Siren'].strip()
-            capital = locale.format( '%d', int(self._dico['capital']), grouping=True)
+            capital = locale.format('%d', int(self._dico['capital']),
+                    grouping=True)
             self._dico.update({
                 'intracommunautary VAT' : VAT_intra,
                 'compact siren' : ''.join(self._dico['Siren'].split()),
                 'capital' : capital, 'Company long form' : long_form, })
-            break; #We select just the first company name 
+            break; #We select just the first company name
 
     def get_name(self):
         return 'company'
@@ -444,7 +473,7 @@ class Bank():
             self._dico.update(zip(header, r))
             iban = ''.join(self._dico[h] for h in [
                     'code', 'agency', 'account number', 'key'])
-            IBAN = ' '.join( [ iban[4*i:4*(i+1)] 
+            IBAN = ' '.join( [ iban[4*i:4*(i+1)]
                         for i in range(len(iban)//4 + len(iban)%4)])
             self._dico.update({ 'IBAN' : IBAN,
                     'short account number' : self._dico['account number'][:5]})
