@@ -1,5 +1,23 @@
 #!/usr/bin/python
 #-*- coding: utf-8 -*-
+
+# Copyright (c) 2013 Eric F <efigue> Figerson
+# Author(s):
+#   Eric F <eric.foss@free.fr>
+
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 import os
 import sys
 import csv
@@ -9,13 +27,16 @@ import decimal
 import glob
 import re
 import time
+import codecs
+
 import argparse
 import config
 from i18n import _
 import bill.gen_tex as gen_tex
-gen_tex.setlocale()
 
+gen_tex.setlocale()
 debug=False
+
 def get_float(str):
     if str:
         return decimal.Decimal(str.replace(',', '.'))
@@ -24,7 +45,6 @@ def get_float(str):
 
 def load_operations(template_file):
     reader = csv.reader(open(template_file, 'r'), delimiter=';', quotechar='"')
-    ops = {}
     op_numbers = {}
     def getNumber(ori_number):
         if ori_number not in op_numbers:
@@ -32,23 +52,23 @@ def load_operations(template_file):
         return op_numbers[ori_number]
 
     for r, row in enumerate(reader):
-        if row:
-            break
+        if row: break
+
+    ops = {}
     for r, row in enumerate(reader):
-        if not row:
-            continue
-        if not row[1]:
+        if not row or not row[1]:
             continue
         op_number = getNumber(row[0])
         if op_number not in ops:
+            unit_period = ''
+            periodicity = ''
             exception = ''
-            if len(row) > 10 and row[10].strip():
-               exception = row[10] 
             if len(row)>9:
-                ops[op_number] = Operation(op_number, row[1], row[8], row[9],
-                        exception)
-            else:
-                ops[op_number] = Operation(op_number, row[1], '', '', exception)
+                unit_period = row[8]
+                periodicity = row[9]
+                if len(row) > 10 and row[10].strip(): exception = row[10]
+            ops[op_number] = Operation(op_number, row[1],
+                    unit_period, periodicity, exception)
         ops[op_number].append(row[2:8])
     return ops
 
@@ -57,13 +77,13 @@ def nb_bill(year, month):
 
 def paid_nb_bill(year, month):
     return nb_bill(year, month) - 2
-        
+
 def paid_nb_month(year, month):
     return 1 + (paid_nb_bill(year, month) -1) % 12
-        
+
 def paid_month_bill(year, month):
     return datetime.date(year, paid_nb_month(year, month),1).strftime('%B')
-        
+
 def paid_year(year, month):
     if month < 3:
         return year - 1
@@ -95,8 +115,8 @@ def last_date_of_month(date):
     return datetime.date(D.year, D.month, 1) + datetime.timedelta(days=int(-1))
 
 def get_day(first_date, year, month, date):
-    if first_date.day > 15: 
-        delta_in_month = first_date.day - last_date_of_month(first_date).day 
+    if first_date.day > 15:
+        delta_in_month = first_date.day - last_date_of_month(first_date).day
         DD = last_date_of_month(datetime.date(year, month, 1))
     else:
         delta_in_month = first_date.day - 1
@@ -105,7 +125,8 @@ def get_day(first_date, year, month, date):
 
 
 class Operation:
-    def __init__(self, op_number, first_date, unit_period, periodicity, exception):
+    def __init__(self, op_number, first_date, unit_period,
+            periodicity, exception):
         self.records = []
         self.op_number = op_number
         self.unit_period = unit_period
@@ -161,7 +182,8 @@ class Operation:
     def generate(self):
         records = {}
         for i, d in enumerate(self.gen_dates()):
-            records[(int(self.op_number), i)] = [ Record(d, r) for r in self.records]
+            records[(int(self.op_number), i)] = [
+                Record(d, r) for r in self.records]
         return records
 
 (CODE, AMOUNT, DATE, LABEL, MARKED) = range(5)
@@ -185,9 +207,9 @@ def concat_2_segments(g, f):
 
 def concat_dumps(merged_dumps_list, f):
     if debug:
-        print ('f=({0}, {1})'.format (getDate(f[0][DATE]), getDate(f[-1][DATE])))
+        print ('f=({0}, {1})'.format(getDate(f[0][DATE]), getDate(f[-1][DATE])))
     if not merged_dumps_list:
-        return merged_dumps_list.append(f) 
+        return merged_dumps_list.append(f)
 
     nb = len(merged_dumps_list)
     j = nb
@@ -226,7 +248,6 @@ def merge_dumps(file_dumps):
         bank_entries.extend(f)
     return bank_entries
 
-import csv, codecs
 
 class UnicodeReader:
     """
@@ -240,11 +261,11 @@ class UnicodeReader:
 
     def next(self):
         return self.__next__()
+
     def __next__(self):
         if not self.lines:
             return []
-        x = self.lines.pop(0).split(self.delimiter)
-        return x
+        return self.lines.pop(0).split(self.delimiter)
 
     def __iter__(self):
         return self
@@ -262,7 +283,8 @@ def loadBank(bankStatement):
         return None
 
     cols_desc = []
-    for title in ['Code Enregistrement', 'Montant', "Date d'opération", 'Libellé']:
+    for title in [
+        'Code Enregistrement', 'Montant', "Date d'opération", 'Libellé']:
         if sys.version_info[0] == 3:
             cols_desc.append(first_line.index(title))
         if sys.version_info[0] == 2:
@@ -296,8 +318,8 @@ def skip_entries(entries, i):
     return end_entries, i
 
 def equiv(e_jnl, e_bk):
-    found = ((e_bk[AMOUNT] == e_jnl[AMOUNT]) and not e_bk[MARKED]\
-            and not e_jnl[MARKED])
+    found = ((e_bk[AMOUNT] == e_jnl[AMOUNT]) and (
+                not e_bk[MARKED] and not e_jnl[MARKED]))
     if found:
         e_bk[MARKED] = True
         e_jnl[MARKED] = True
@@ -354,7 +376,7 @@ def get_valid_jnl_slice(bank_entries, jnl_entries):
             break
     return jnl_entries[j_begin:j_end]
 
-class WriteGenerator:            
+class WriteGenerator:
     def __init__(self, year, bank_entries):
         self._bank_entries = bank_entries
         self._year = year
@@ -362,17 +384,17 @@ class WriteGenerator:
         self._orphelins = []
         for e in self._bank_entries:
             e[LABEL] = ' '.join(e[LABEL].split('\n')[0].split()[1:-1])
-            
+
     def compute(self):
         essence = ( 'Essence',
                 ('625000', 'Voyage et déplacement'),
                 ('401100', 'Fournisseurs - Achats de biens et services'),
                 ('512100', 'Banque'))
-        peage = ('Péage', essence[1], essence[2], essence[3]) 
+        peage = ('Péage', essence[1], essence[2], essence[3])
         categories = {
             peage: ('COFIROUTE',),
             essence: ('PAREA CONFL DAC', 'PAREA CONFL STA', 'REL. CLAIR BOIS',
-                    'STATION BP ROC', 'COFIROUTE', 'E.LECLERC DAC', 'ESSO PAVE'),
+                'STATION BP ROC', 'COFIROUTE', 'E.LECLERC DAC', 'ESSO PAVE'),
         }
         self._rows = []
         def get_row(cat, entry, transit_account=True, bill_time='facture'):
@@ -424,7 +446,7 @@ class Journal:
 
     def merge(self):
         for k in sorted(self.ops.keys()):
-           self.entries.extend(self.ops[k]) 
+           self.entries.extend(self.ops[k])
 
     def cleanup(self):
         self.merge()
@@ -434,7 +456,7 @@ class Journal:
             if k in tmp:
                 e.row.append('DUPLICATE')
             tmp[k] = e
-        self.entries = [ e for e in self.entries if not e.row[-1] == 'DUPLICATE']
+        self.entries = [e for e in self.entries if not e.row[-1] == 'DUPLICATE']
         self.sort()
 
     def corpTaxWritting(self, tax):
@@ -451,8 +473,9 @@ class Journal:
         gross_profit = 0
         for e in self.entries:
             account = int(e.row[COMPTES])
-            if (account >= 600000 and account < 700000 and account != 695000)\
-                           or account == 706000:
+            if account == 695000:
+                continue
+            if (account >= 600000 and account < 700000) or account == 706000:
                 gross_profit += get_float(e.row[CREDIT]) - get_float(e.row[DEBIT])
         return gross_profit
 
@@ -626,7 +649,8 @@ def main(orga, year):
         sys.stderr.write(_('Please give a year after 2010 (%s given)\n') % year)
         return 2
     journal = Journal(orga, year)
-    tmpl_file = os.path.join(get_account_dir(orga), str(year), 'journal_tmpl.csv')
+    tmpl_file = os.path.join(get_account_dir(orga),
+            str(year), 'journal_tmpl.csv')
     journal.load_operations(tmpl_file)
     journal.cleanup()
     journal.checkBank()
