@@ -1,13 +1,14 @@
 #!/bin/bash
 script_root=$(dirname $(realpath $0))
 app_root=$(dirname $script_root)
+export PYTHONPATH=$PYTHONPATH:$app_root
 py_root=$(dirname $(ls $app_root/*/config.py))
 domain=$(basename $py_root)
 lang=fr
 country=FR
 LC=${lang}_${country}
 po_dir=$app_root/po
-mo_dir=$py_root/share/i18n/$LC/LC_MESSAGES
+mo_dir=$py_root/locale/$LC/LC_MESSAGES
 
 if [ "$1" = "sav" ]; then
     rm -f $po_dir/$lang.po.bkup
